@@ -2,22 +2,24 @@
 #
 # SPDX-License-Identifier: MPL-2.0
 from typing import Union
+
 import structlog
 
 from openstef.enums import MLModelType
+from openstef.model.regressors.custom_regressor import is_custom_type, load_custom_model
 from openstef.model.regressors.lgbm import LGBMOpenstfRegressor
+from openstef.model.regressors.linear import LinearOpenstfRegressor
 from openstef.model.regressors.regressor import OpenstfRegressor
 from openstef.model.regressors.xgb import XGBOpenstfRegressor
 from openstef.model.regressors.xgb_quantile import XGBQuantileOpenstfRegressor
-from openstef.model.regressors.linear import LinearOpenstfRegressor
-from openstef.model.regressors.custom_regressor import load_custom_model, is_custom_type
 
 logger = structlog.get_logger(__name__)
 try:
     from openstef.model.regressors.proloaf import OpenstfProloafRegressor
 except ImportError:
     logger.warning(
-        "Proloaf not available, switching to xgboost! See Readme how to install proloaf dependencies"
+        "Proloaf not available, switching to xgboost! See Readme how to install proloaf"
+        " dependencies"
     )
     OpenstfProloafRegressor = XGBOpenstfRegressor
 
@@ -151,7 +153,7 @@ class ModelCreator:
             raise NotImplementedError(
                 f"No constructor for '{model_type}', "
                 f"valid model_types are: {valid_types}"
-                f"or import a custom model"
+                "or import a custom model"
             ) from e
 
         # only pass relevant arguments to model constructor to prevent warnings
