@@ -4,7 +4,7 @@
 
 """create_solar_forecast
 This module contains the CRON job that is periodically executed to make
-prognoses of solar features that are usefull for splitting the load in solar and
+prognoses of solar features that are useful for splitting the load in solar and
 wind contributions.
 Example:
     This module is meant to be called directly from a CRON job. A description of
@@ -29,21 +29,19 @@ from openstef.tasks.utils.taskcontext import TaskContext
 PV_COEFS_FILEPATH = PROJECT_ROOT / "openstef" / "data" / "pv_single_coefs.csv"
 
 
-def make_solar_predicion_pj(pj, context):
-    """Make a solar prediction for a spcecific prediction job.
+def make_solar_prediction_pj(pj, context, radius=30, peak_power=180961000.0):
+    """Make a solar prediction for a specific prediction job.
 
     Args:
         pj: (dict) prediction job
     """
-    pj["radius"] = 30
-    pj["peak_power"] = 180961000.0
     context.logger.info("Get solar input data from database")
     # pvdata is only stored in the prd database
     solar_input = context.database.get_solar_input(
         (pj["lat"], pj["lon"]),
         pj["horizon_minutes"],
         pj["resolution_minutes"],
-        radius=pj["radius"],
+        radius=radius,
         sid=pj["sid"],
     )
 
@@ -58,8 +56,8 @@ def make_solar_predicion_pj(pj, context):
     )
 
     # if the forecast is for a region, output should be scaled to peak power
-    if (pj["radius"] != 0) and (not np.isnan(pj["peak_power"])):
-        power = pj["peak_power"] / max(solar_input.aggregated) * power
+    if (radius != 0) and (not np.isnan(peak_power)):
+        power = peak_power / max(solar_input.aggregated) * power
     context.logger.info("Store solar prediction in database")
     power["pid"] = pj["id"]
     power["type"] = "solar"
@@ -84,7 +82,7 @@ def combine_forecasts(forecasts, combination_coefs):
 
     # Add subset parameters to df
     # Identify which parameters should be used to define subsets based on the
-    # combinationcoefs
+    # combination coefs
     subset_columns = [
         "tAhead",
         "hForecasted",
@@ -222,11 +220,12 @@ def main(config=None, database=None):
 
     if database is None or config is None:
         raise RuntimeError(
-            "Please specifiy a configmanager and/or database connection object. These can be found in the openstef-dbc package."
+            "Please specify a configmanager and/or database connection object. These"
+            " can be found in the openstef-dbc package."
         )
 
     with TaskContext(taskname, config, database) as context:
-        context.logger.info("Querying wind prediction jobs from database")
+        context.logger.info("Querying solar prediction jobs from database")
         prediction_jobs = context.database.get_prediction_jobs_solar()
         num_prediction_jobs = len(prediction_jobs)
 
@@ -245,7 +244,7 @@ def main(config=None, database=None):
             )
 
         PredictionJobLoop(context, prediction_jobs=prediction_jobs).map(
-            make_solar_predicion_pj, context
+            make_solar_prediction_pj, context
         )
 
 
@@ -374,7 +373,7 @@ def apply_fit_insol(data, add_to_df=True, hours_delta=None, polynomial=False):
         return coefs[0] * values + coefs[1]
 
     def second_order_poly(coefs, values):
-        return coefs[0] * values ** 2 + coefs[1] * values + coefs[2]
+        return coefs[0] * values**2 + coefs[1] * values + coefs[2]
 
     # Define function to be minimized and subsequently minimize this function
     if polynomial:
